@@ -1,8 +1,8 @@
 import contextlib
 import os
 import os.path
-import cStringIO as StringIO
 
+from httpretty import HTTPretty
 import mock
 import requests
 
@@ -60,6 +60,7 @@ def mocked_suds_client(waterml_version, mocked_service_calls):
 
             for service_call, filename in mocked_service_calls.iteritems():
                 open_file = open_files[filename]
+
                 def _func(*args, **kwargs):
                     return open_file.read()
                 setattr(client.service, service_call, _func)
@@ -68,15 +69,43 @@ def mocked_suds_client(waterml_version, mocked_service_calls):
                 yield
 
 
+@contextlib.contextmanager
+def mocked_url(url, response_file_path, methods=None):
+    """mocks the underlying python sockets library to return a given file's
+    content
+    """
+    # if environment variable is set, then don't mock the tests just grab files
+    # over the network. Example:
+    #    env ULMO_DONT_MOCK_TESTS=1 py.test
+    if os.environ.get('ULMO_DONT_MOCK_TESTS', False):
+        yield
+
+    else:
+        with open(get_test_file_path(response_file_path)) as f:
+            response = f.read()
+
+        HTTPretty.enable()
+
+        if methods is None:
+            methods = ['GET', 'POST']
+
+        for method in methods:
+            request_class = getattr(HTTPretty, method)
+            HTTPretty.register_uri(request_class, url, body=response)
+
+        yield
+        HTTPretty.disable()
+
+
 def _mock_request_side_effect(url_files):
     def _side_effect(url, *args, **kwargs):
         mock_response = requests.Response()
-        mock_response.request = requests.Request(url, *args, **kwargs)
+        mock_response.request = requests.Request('GET', url, *args, **kwargs).prepare()
         mock_response.status_code = 200
         mock_response.url = url
         mock_response.mocked = True
         if isinstance(url_files, dict):
-            mock_response.raw = url_files.get(mock_response.request.full_url)
+            mock_response.raw = url_files.get(mock_response.request.url)
         elif hasattr(url_files, 'read'):
             mock_response.raw = url_files
 
